// Nelder Mead minimization for a function of 4 variables
// Minimizes the function f(x).  Uses several functions
// f(vector<double>) = the function to minimize.  Returns a scalar
// VMean(vector<vector<double> > ) = returns the mean of a vector
// VAdd(vector<double>, vector<double> ) = adds two vectors, returns a vector
// VSub(vector<double>, vector<double> ) = subtracts two vectors, returns a vector
// VMult(vector<double>, double ) = multiplies a vector by a constant
// NelderMead(double (*f)(vector<double>), int, double,double, double, vector<vector<double> >) =
//		the Nelder Mead algorithm. Returns a scalar.
// To use the Nelder Mead algorithm, change the function f(x) to the function you want.
// Then change the vertices in int main() so that they contain your starting values.
// If your function is of dimension N, you need N+1 vertices
// The change the arguments of the NelderMead function to pass the parameters you need.

// by Fabrice Douglas Rouah, FRouah.com and Volopta.com

#include "stdafx.h"
#include <iostream>
#include <fstream>
#include <iomanip>
#include <vector>
#include <cmath>
#include <algorithm>
using namespace std;
#include "NelderMead.h"

// Objective Function to minimize
double ObjFun(vector<double> x) {
	return pow(x[0]-3215.14, 2) + pow(x[1]+1500, 2) + pow(x[2]-0.003, 2) + pow(x[3]+0.000015, 2.0);
}

int main() {
	// Settings for Nelder Mead Algorithm
	NMinputs NMin;
	NMin.N = 4;					// Number of parameters in f(x) to find
	NMin.MaxIters = 1e4 ;		// Maximum number of iterations
	NMin.Tolerance = 1e-20;		// Tolerance on best and worst function values
	int N = NMin.N;

	// Starting values (vertices) in vector form.   Add more as needed
	vector<vector<double> > start(N,vector<double>(N+1));

	// Vertice 0	     Vertice 1		     Vertice 2		    Vertice 3		    Vertice 4
	//----------------------------------------------------------------------------------------------
	start[0][0]=-9.1;	start[0][1]=-9.1;	start[0][2]=-5.0;	start[0][3]=99.2;	start[0][4]=-9.9;
	start[1][0]=-1.5;	start[1][1]=-1.4;	start[1][2]=-1.6;	start[1][3]=-1.5;	start[1][4]=-1.4;
	start[2][0]=99.2;	start[2][1]=49.3;	start[2][2]=39.2;	start[2][3]=98.2;	start[2][4]=11.1;
	start[3][0]=9.1;	start[3][1]=-9.1;	start[3][2]= 9.2;	start[3][3]= 9.1;	start[3][4]= 9.5;
	CNelderMead NM;
	vector<double> Output = NM.NelderMeadFunction(ObjFun,NMin,start);
	cout << "Nelder Mead Minimization" << endl;
	cout << "------------------------" << endl;
	for (int i=0; i<=N-1; i++)
		cout << "Parameter x(" << i << ") =  " << Output[i] << endl;
	cout << "-----------------------------------------" << endl;
	cout << "Value of Objective Function = " << Output[N] << endl;
	cout << "Number of Iterations = " << Output[N+1] << endl;
	cout << "-----------------------------------------" << endl;
}
